import '../../pages/makeup/camera/camera'
import { createSelectorQueryMock } from '../wx'

import mockDefine from '../../define'
import mockStatus from '../../pfEngine/utils/status'
import { getLooksResponse, getListByEffectResponse } from '../response'
import expect from 'expect'

const page = global.wxPageInstance
const initialPageData = JSON.parse(JSON.stringify(page.data))
const pfMakeupApi = require('../../pfEngine/api/pfMakeup')
const mockCanvasNode = {
  node: {
    getContext: jest.fn(() => ({
      canvas: { width: 300, height: 300 }
    }))
  }
}

describe('page life cycle', () => {
  it('onLoad', () => {
    page.onLoad({})
    expect(getApp().globalData.domain).toBe('new_domain')
    expect(getApp().globalData.dataType).toBe('new_dataType')
  })
  it('onShow', () => {
    page.onShow()
  })
  it('onHide', () => {
    page.onHide()
  })
  it('onReady', () => {
    wx.createSelectorQuery = createSelectorQueryMock(mockCanvasNode)
    page.onReady()
    expect(page.canvas).not.toBeNull()
    expect(pfMakeupApi.init).toBeCalledWith(
      expect.objectContaining({
        canvas: expect.anything(),
        callback: expect.any(Function),
        isPhoto: expect.any(Boolean)
      })
    )
  })
  it('onUnload', () => {
    page.onUnload()
    expect(page.canvas).toBeNull()
  })
  afterAll(() => {
    jest.clearAllMocks()
  })
})

describe('callback', () => {
  beforeAll(() => {
    // set global pfMakeup
    page._initPfMakeup()
  })
  it('init callback (camera mode)', () => {
    getApp().globalData.defaultPhotoMode = false
    const spyUpdateIntensity = jest.spyOn(page, 'updateIntensity').mockImplementationOnce(() => {})
    const spyCamera = jest.spyOn(page, 'onCameraModeClick').mockImplementationOnce(() => {})
    const spyPhoto = jest.spyOn(page, 'onPhotoModeClick').mockImplementationOnce(() => {})
    page.initCallback(mockStatus.STATUS_SUCCESS)
    expect(spyUpdateIntensity).toBeCalled()
    expect(spyCamera).toBeCalled()
    expect(spyPhoto).not.toBeCalled()
  })
  it('init callback (photo mode)', () => {
    getApp().globalData.defaultPhotoMode = true
    const spyUpdateIntensity = jest.spyOn(page, 'updateIntensity').mockImplementationOnce(() => {})
    const spyCamera = jest.spyOn(page, 'onCameraModeClick').mockImplementationOnce(() => {})
    const spyPhoto = jest.spyOn(page, 'onPhotoModeClick').mockImplementationOnce(() => {})
    page.initCallback(mockStatus.STATUS_SUCCESS)
    expect(spyUpdateIntensity).toBeCalled()
    expect(spyCamera).not.toBeCalled()
    expect(spyPhoto).toBeCalled()
  })
  afterEach(() => {
    jest.clearAllMocks()
  })
})

describe('sku type click', () => {
  beforeAll(() => {
    // set global pfMakeup
    page._initPfMakeup()
  })
  const button = type => ({ currentTarget: { dataset: { type } } })
  it('look click', async () => {
    jest.spyOn(page, 'loadAllLook').mockImplementationOnce(() => Promise.resolve(getLooksResponse))
    await page.onSkuTypeClick(button(mockDefine.MAKEUP_TYPE.LOOK.type))
    expect(page.data.products).toEqual(getLooksResponse)
    expect(page.data.skuTypeList.selected).toEqual({ name: '主题妆容', type: 'Look' })
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(false)
    expect(page.data.skuTypeIntensityState.sliderValue).toBe(undefined)
    expect(page.data.skuTypeIntensityState.subType).toBeFalsy()
  })
  it('lipstick click', async () => {
    jest
      .spyOn(page, 'loadAllListByEffect')
      .mockImplementationOnce(() => Promise.resolve(getListByEffectResponse))
    await page.onSkuTypeClick(button(mockDefine.MAKEUP_TYPE.LIPSTICK.type))
    expect(page.data.products).toEqual(getListByEffectResponse)
    expect(page.data.skuTypeList.selected).toEqual({ name: '唇彩', type: 'lipstick' })
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(false)
    expect(page.data.skuTypeIntensityState.sliderValue).toBe(undefined)
    expect(page.data.skuTypeIntensityState.subType).toBeFalsy()
  })
  it('lipstick reset', async () => {
    await page.onSkuTypeClick(button(mockDefine.MAKEUP_TYPE.LIPSTICK.type))
    expect(page.data.products).toEqual([])
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(false)
  })
  it('lipstick click (with appiled lipstick)', async () => {
    const mockLipstick = {
      productGuid: 'BCC-20502_20200517_LS_01',
      skuGuid: 'BCC-20502_20200517_LS_01_01',
      paletteGuid: 'BCC-20502_20200517_LS_01_01',
      patternGuid: 'patten_lipstick_02'
    }
    page.data.tryOnList.lipstick = mockLipstick
    pfMakeupApi.getIntensities.mockImplementationOnce(() => ({
      lipstick: 50
    }))
    jest
      .spyOn(page, 'loadAllListByEffect')
      .mockImplementationOnce(() => Promise.resolve(getListByEffectResponse))
    await page.onSkuTypeClick(button(mockDefine.MAKEUP_TYPE.LIPSTICK.type))
    expect(page.data.products).toEqual(getListByEffectResponse)
    expect(page.data.selectedProduct).toEqual(getListByEffectResponse[0])
    expect(page.data.skus).toBe(getListByEffectResponse[0].skus)
    expect(page.data.skuTypeList.selected).toEqual({ name: '唇彩', type: 'lipstick' })
    expect(page.data.patternGuids).toEqual(['patten_lipstick_02'])
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(true)
    expect(page.data.skuTypeIntensityState.sliderValue).toBe(50)
    expect(page.data.skuTypeIntensityState.subType).toBeFalsy()
  })
  it('lipstick reset (with appiled lipstick)', async () => {
    await page.onSkuTypeClick(button(mockDefine.MAKEUP_TYPE.LIPSTICK.type))
    expect(page.data.products).toEqual([])
    expect(page.data.skus).toEqual([])
    expect(page.data.skuTypeList.selected).toBeNull()
    expect(page.data.patternGuids).toEqual([])
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(false)
    expect(page.data.tryOnList.lipstick).not.toBeNull()
  })
})

describe('product click', () => {
  const button = index => ({
    currentTarget: { dataset: { productIdx: index, productGuid: getListByEffectResponse[index] } }
  })
  it('lipstick product click', async () => {
    page.data = JSON.parse(JSON.stringify(initialPageData))
    page.data.products = getListByEffectResponse
    page.data.skuTypeList.selected = { name: '唇彩', type: 'lipstick' }
    await page.onProductClick(button(0))
    expect(page.data.selectedProduct).toEqual(getListByEffectResponse[0])
    expect(page.data.skus).toBe(getListByEffectResponse[0].skus)
  })
  it('look product click', async () => {
    page.data = JSON.parse(JSON.stringify(initialPageData))
    page.data.products = getLooksResponse
    page.data.skuTypeList.selected = { name: '主题妆容', type: 'Look' }
    await page.onProductClick(button(0))
    expect(pfMakeupApi.clearEffects).toBeCalledWith(expect.any(Array), expect.any(Function))
    expect(page.data.selectedProduct).toEqual(getLooksResponse[0])
    expect(page.data.skus).toEqual([])
  })
})

describe('sku click', () => {
  const mockLipstickButton = {
    skuType: 'lipstick',
    productGuid: 'BCC-20502_20200517_LS_01',
    skuGuid: 'BCC-20502_20200517_LS_01_01',
    paletteGuid: 'BCC-20502_20200517_LS_01_01'
  }
  const mockLipstick = {
    productGuid: 'BCC-20502_20200517_LS_01',
    skuGuid: 'BCC-20502_20200517_LS_01_01',
    paletteGuid: 'BCC-20502_20200517_LS_01_01',
    patternGuid: 'patten_lipstick_02'
  }
  const button = { currentTarget: { dataset: mockLipstickButton } }
  it('lipstick sku click', async () => {
    pfMakeupApi.getIntensities.mockImplementationOnce(() => ({
      lipstick: 50
    }))
    page.data = JSON.parse(JSON.stringify(initialPageData))
    page.data.products = getListByEffectResponse
    page.data.skuTypeList.selected = { name: '唇彩', type: 'lipstick' }
    page.data.selectedProduct = getListByEffectResponse[0]
    page.data.skus = getListByEffectResponse[0].skus
    await page.onSkuClick(button)
    expect(pfMakeupApi.applyMakeupBySkus).toBeCalledWith(expect.any(Array), expect.any(Function))
    expect(page.data.tryOnList.lipstick).toEqual(mockLipstick)
    expect(page.data.patternGuids).toEqual(['patten_lipstick_02'])
    expect(page.data.skuTypeIntensityState.isEnabled).toBe(true)
    expect(page.data.skuTypeIntensityState.sliderValue).toBe(50)
    expect(page.data.skuTypeIntensityState.subType).toBeFalsy()
  })
})

describe('update intensity', () => {
  beforeAll(() => {
    // set global pfMakeup
    page._initPfMakeup()
  })
  it('lipstick intensity', () => {
    pfMakeupApi.getIntensities.mockImplementationOnce(() => ({
      lipstick: 30
    }))
    page.updateIntensity(30, false, mockDefine.MAKEUP_TYPE.LIPSTICK.type)
    expect(page.data.skuTypeIntensityState.sliderValue).toBe(30)
  })
  it('skin smooth intensity', () => {
    pfMakeupApi.getIntensities.mockImplementationOnce(() => ({
      skinSmooth: 40
    }))
    page.updateIntensity(40, false, mockDefine.FEATURE.SKIN_SMOOTH.type)
    expect(page.data.featureSlider.values.skinSmooth).toBe(40)
  })
  it('eye wear intensity', () => {
    pfMakeupApi.getIntensities.mockImplementationOnce(() => ({
      eye_wear: {
        pd: 70
      }
    }))
    page.updateIntensity({ pd: 70 }, false, mockDefine.MAKEUP_TYPE.EYE_WEAR.type)
    expect(page.data.skuTypeIntensityState.sliderValue.pd).toBe(70)
  })
})
