import logger from '../../pfEngine/utils/logger'
import { measureText } from '../../pfEngine/utils/util'
import { PRODUCT_CATEGORY } from '../../define'

const app = getApp()

Component({
  properties: {
    productsByCategory: {
      type: Object,
      value: null
    },
    categories: {
      type: Array,
      value: []
    },
    startSaving: {
      type: Boolean,
      value: false
    }
  },
  data: {
    canvasId: 'offScreenCanvas',
    canvasWidth: 0,
    canvasHeight: 0,
    unit: 0,
    mainColor: 'rgba(67, 165, 220, 1)'
  },
  nowBottomY: 0,
  resultFilePath: '',
  isSaving: false,
  observers: {
    startSaving: async function(startSaving) {
      if (
        this.isSaving ||
        !startSaving ||
        !this.data.productsByCategory ||
        !this.data.categories.length
      ) {
        return
      }

      try {
        wx.showLoading({
          title: '正在储存'
        })
        if (!this.resultFilePath) {
          await this._setDataAndDrawCanvas()
          const res = await this._canvasToTempFilePath()
          this.resultFilePath = res.tempFilePath
        }
        await this._saveToAlbum(this.resultFilePath)
        this._onSaveSuccess()
      } catch (error) {
        logger.error('[saveImageCanvas]fail:', error)
        this._onSaveFail(this._getErrorMessage(error))
      }
    }
  },
  lifetimes: {
    attached: function() {
      this.setData({
        unit: app.globalData.systemInfo.windowWidth / 320
      })
    }
  },
  methods: {
    _onSaveSuccess: function() {
      this._onSaveComplete()
      this.triggerEvent('save-success')
    },
    _onSaveFail: function(errMsg) {
      this._onSaveComplete()
      const eventDetail = { error: errMsg }
      this.triggerEvent('save-fail', eventDetail)
    },
    _onSaveComplete: function() {
      this.setData({ startSaving: false })
      this.isSaving = false
      wx.hideLoading()
    },
    _getErrorMessage: function(error = '') {
      if (typeof error === 'string') {
        return error
      }
      if (error.errMsg) {
        return error.errMsg
      }
      if (error instanceof Error && error.message) {
        return error.message
      }
      return error
    },
    _setDataAndDrawCanvas: function() {
      const { unit, productsByCategory, categories } = this.data
      logger.log(`[_setDataAndDrawCanvas]unit=${unit}`)
      logger.log(`[_setDataAndDrawCanvas]productsByCategory:`, productsByCategory)
      logger.log(`[_setDataAndDrawCanvas]categories:`, categories)

      const productTotalLength = this._countProducts(productsByCategory)

      const paddingTopBottom = 18 * unit * 2
      const categoryTitleMarginTopBottom = (23.7 + 10) * unit
      const productListTitleFontSize = 13.6 * unit
      const productListTitleMarginBottom = 20 * unit

      return new Promise((resolve, reject) => {
        this.setData(
          {
            canvasWidth: app.globalData.systemInfo.windowWidth,
            canvasHeight:
              paddingTopBottom +
              categoryTitleMarginTopBottom * categories.length +
              productListTitleFontSize +
              productListTitleMarginBottom +
              83 * unit * productTotalLength
          },
          () => {
            this._drawOffscreenCanvas()
              .then(() => resolve())
              .catch(error => reject(error))
          }
        )
      })
    },
    _saveToAlbum: function(filePath) {
      logger.log(`[_saveToAlbum]filePath=${filePath}`)
      wx.showLoading({
        title: '正在储存'
      })
      return this._saveImageToPhotosAlbum(filePath).catch(error => {
        const errMsg = this._getErrorMessage(error)
        if (errMsg.includes('auth')) {
          return this._checkSavePhotoAuthAndRetrySave()
        }
        throw Error(errMsg)
      })
    },
    _checkSavePhotoAuthAndRetrySave: function() {
      return this._checkSavePhotoAuth()
        .then(isAuth => isAuth || this._openSetting4SavePhotoAuth())
        .then(() => this._saveToAlbum(this.resultFilePath))
    },
    _canvasToTempFilePath: function() {
      const { canvasId, canvasWidth, canvasHeight } = this.data
      const pixelRatio = app.globalData.systemInfo.pixelRatio

      return new Promise((resolve, reject) => {
        wx.canvasToTempFilePath(
          {
            canvasId,
            x: 0,
            y: 0,
            width: canvasWidth * pixelRatio,
            height: canvasHeight * pixelRatio,
            destWidth: canvasWidth * pixelRatio,
            destHeight: canvasHeight * pixelRatio,
            success: res => resolve(res),
            fail: error => reject(error)
          },
          this
        )
      })
    },
    _saveImageToPhotosAlbum: function(filePath) {
      return new Promise((resolve, reject) => {
        wx.saveImageToPhotosAlbum(
          {
            filePath,
            success: res => resolve(res),
            fail: error => reject(error)
          },
          this
        )
      })
    },
    _checkSavePhotoAuth: function() {
      return new Promise(resolve => {
        wx.getSetting({
          success: res => {
            logger.log('[getSetting]res:', res)
            !res.authSetting && logger.error('[getSetting] res.authSetting is empty')

            resolve(res.authSetting && res.authSetting['scope.writePhotosAlbum'])
          },
          fail: error => {
            logger.error('[getSetting]fail:', error)
            resolve(false)
          }
        })
      })
    },
    _openSetting4SavePhotoAuth: function() {
      return new Promise((resolve, reject) => {
        wx.showModal({
          title: '相片存储权限',
          content: '请确认相簿权限已开启',
          success: res => {
            if (res.cancel) {
              reject(Error('save-photo authorization was canceled'))
              return
            }
            wx.openSetting({
              success: res => logger.log('[openSetting]success:', res),
              fail: error => logger.error('[openSetting]fail:', error),
              complete: () => resolve()
            })
          }
        })
      })
    },
    _countProducts: function(productsByCategory) {
      let count = 0
      Object.values(productsByCategory).forEach(products => (count += products.length))

      return count
    },
    _getCanvasContext: function(id) {
      return new Promise(resolve => {
        this.createSelectorQuery()
          .in(this)
          .select(`#${id}`)
          .context(res => resolve(res.context))
          .exec()
      })
    },
    _drawOffscreenCanvas: async function() {
      // eslint-disable-next-line no-useless-catch
      try {
        this.nowBottomY = 0
        const context = await this._getCanvasContext(this.data.canvasId)
        const productIdToImageInfo = await this._getProductIdToImageInfoMap(
          this.data.productsByCategory
        )
        logger.log('[_drawOffscreenCanvas]productIdToImageInfo:', productIdToImageInfo)

        this._drawFillWhite(context, this.data.canvasWidth, this.data.canvasHeight)
        this._drawProductTitle(context)
        this._drawProductList(context, productIdToImageInfo)
        return this._actuallyDraw(context)
      } catch (error) {
        throw error
      }
    },
    _getProductIdToImageInfoMap: function(productsByCategory) {
      return new Promise((resolve, reject) => {
        const imageUrls = this._getProductImageUrls(productsByCategory)
        logger.log('[_getProductIdToImageInfoMap]imageUrls:', imageUrls)

        Promise.all(imageUrls.map(img => this._getProductImageInfo(img)))
          .then(imageInfos => {
            const productIdToImageInfo = {}
            imageInfos.forEach(info => (productIdToImageInfo[info.productId] = info))

            resolve(productIdToImageInfo)
          })
          .catch(error => reject(error))
      })
    },
    _getProductImageUrls: function(productsByCategory) {
      const imageUrls = []
      Object.values(productsByCategory).forEach(products => {
        products.forEach(product => {
          imageUrls.push({ productId: product.productId, image: product.image })
        })
      })
      return imageUrls
    },
    _getProductImageInfo: function({ productId, image }) {
      return new Promise(function(resolve, reject) {
        wx.getImageInfo({
          src: image,
          success: res => {
            resolve({
              productId,
              path: res.path,
              inputSrc: image
            })
          },
          fail: error => reject(error)
        })
      })
    },
    _drawFillWhite: function(canvas, width, height) {
      canvas.setFillStyle('white')
      canvas.rect(0, 0, width, height)
      canvas.fill()
    },
    _drawProductTitle: function(canvas) {
      const { unit, mainColor } = this.data

      this.nowBottomY += 18 * unit
      const marginTop = this.nowBottomY
      const titleLineLength = 35.7 * unit
      const paddingHorizonLength = 92.3 * unit
      const titlePaddingHorizonLength = 4.7 * unit
      const linePositionY = 10 * unit

      canvas.strokeStyle = mainColor
      canvas.lineWidth = 1
      canvas.beginPath()
      canvas.moveTo(paddingHorizonLength, marginTop + linePositionY)
      canvas.lineTo(paddingHorizonLength + titleLineLength, marginTop + linePositionY)
      canvas.closePath()
      canvas.stroke()

      const titleText = '产品推荐'
      const textSize = 13.6 * unit
      canvas.font = 'normal bold 14px NotoSansSC-Regular'
      canvas.setFontSize(textSize)
      canvas.fillStyle = mainColor
      canvas.fillText(
        titleText,
        paddingHorizonLength + titleLineLength + titlePaddingHorizonLength,
        marginTop + textSize
      )

      canvas.strokeStyle = mainColor
      canvas.lineWidth = 1
      canvas.beginPath()
      canvas.moveTo(
        paddingHorizonLength +
          titleLineLength +
          titlePaddingHorizonLength * 2 +
          measureText(titleText, textSize),
        marginTop + linePositionY
      )
      canvas.lineTo(
        paddingHorizonLength +
          titleLineLength * 2 +
          titlePaddingHorizonLength * 2 +
          measureText(titleText, textSize),
        marginTop + linePositionY
      )
      canvas.closePath()
      canvas.stroke()

      this.nowBottomY += 20 * unit

      //eBug:WM1202009-0007 canvas.draw(true) is a workaround for wrong fillStyle/strokeStyle.
      canvas.draw(true)

    },
    _drawProductList: function(canvas, imageInfos) {
      this.data.categories.forEach(({ value: category }) => {
        this._drawProductListByCategory(category, canvas, imageInfos)
      })
    },
    _drawProductListByCategory: function(category, canvas, imageInfos) {
      const { unit, productsByCategory } = this.data

      const products = productsByCategory[category]
      if (!products || !products.length) return

      const rectWidth = 71 * unit
      const marginLeft = 24 * unit

      this.nowBottomY += 23.7 * unit

      canvas.font = '16px NotoSansSC-Regular'
      canvas.setFontSize(16 * unit)

      canvas.fillStyle = 'black' // All use black
      canvas.fillText(PRODUCT_CATEGORY[category].label, marginLeft, this.nowBottomY)

      this.nowBottomY += 10 * unit

      products.forEach(product => {
        const path = imageInfos[product.productId] && imageInfos[product.productId].path
        canvas.drawImage(
          path,
          marginLeft + 1 * unit,
          this.nowBottomY + 1 * unit,
          rectWidth - 2 * unit,
          rectWidth - 2 * unit
        )
        this._dealWords({
          ctx: canvas,
          fontSize: 11 * unit,
          word: product.productName,
          maxWidth: 192 * unit, // The maximum length for each line
          x: marginLeft + rectWidth + 9 * unit, // X-position for text to show
          y: this.nowBottomY, // Y-position for text to show (We take the position of the image for start)
          maxLine: 3, // The maximum line count for showing name
          lineHeight: 16.3 * unit, // The height of each line.
          imageHeight: rectWidth
        })
        this._drawRadiusRect(
          canvas,
          marginLeft,
          this.nowBottomY,
          marginLeft + rectWidth,
          this.nowBottomY + rectWidth
        )
        this.nowBottomY += 83 * unit
      })
    },
    _drawRadiusRect: function(canvas, x1, y1, x2, y2) {
      const radius = 5

      canvas.setLineWidth(1)
      canvas.strokeStyle = 'black'
      canvas.fillStyle = 'black'

      canvas.moveTo(x2 - radius, y2)
      canvas.lineTo(x1 + radius, y2)

      canvas.moveTo(x1, y2 - radius)
      canvas.lineTo(x1, y1 + radius)

      canvas.moveTo(x1 + radius, y1)
      canvas.lineTo(x2 - radius, y1)

      canvas.moveTo(x2, y1 + radius)
      canvas.lineTo(x2, y2 - radius)

      canvas.stroke()

      canvas.beginPath()
      canvas.arc(x2 - radius, y2 - radius, radius, 0, 0.5 * Math.PI)
      canvas.stroke()
      canvas.beginPath()
      canvas.arc(x1 + radius, y2 - radius, radius, 0.5 * Math.PI, Math.PI)
      canvas.stroke()
      canvas.beginPath()
      canvas.arc(x1 + radius, y1 + radius, radius, Math.PI, 1.5 * Math.PI)
      canvas.stroke()
      canvas.beginPath()
      canvas.arc(x2 - radius, y1 + radius, radius, 1.5 * Math.PI, 2 * Math.PI)
      canvas.stroke()
    },
    _actuallyDraw: function(canvas) {
      return new Promise(resolve => {
        canvas.draw(true, () => resolve())
      })
    },

    // 3rd party
    _dealWords: function(options) {
      options.ctx.font = '16px NotoSansSC-Regular'
      options.ctx.setFontSize(options.fontSize)
      options.ctx.fillStyle = 'black'
      const allRow = Math.ceil(measureText(options.word, options.fontSize) / options.maxWidth) // Actually we can have allRow line
      const count = allRow >= options.maxLine ? options.maxLine : allRow // Take the small one between the maxLine and allRow to determine the round count of for loop

      // Move start position to the middle of image
      options.y +=
        options.imageHeight / 2 +
        options.lineHeight -
        (options.lineHeight / 2) * allRow -
        options.lineHeight / 4

      let endPos = 0 // the cutting point (start point) for each round
      for (let j = 0; j < count; j++) {
        const nowStr = options.word.slice(endPos, options.word.length) // The rest of string for now.
        let rowWid = 0 // The text width of each round
        if (measureText(nowStr, options.fontSize) > options.maxWidth) {
          // If too long, starting to cut
          for (let m = 0; m < nowStr.length; m++) {
            rowWid += measureText(nowStr[m], options.fontSize) // The total width of nowStr
            if (rowWid > options.maxWidth) {
              if (j === options.maxLine - 1) {
                // Is the last line
                options.ctx.fillText(
                  nowStr.slice(0, m - 1) + '...',
                  options.x,
                  options.y + j * options.lineHeight
                )
              } else {
                options.ctx.fillText(
                  nowStr.slice(0, m),
                  options.x,
                  options.y + j * options.lineHeight
                )
              }
              endPos += m // Cutting point for next round
              break
            }
          }
        } else {
          // If nowStr < maxWidth, output it directly
          options.ctx.fillText(
            nowStr.slice(0, nowStr.length),
            options.x,
            options.y + j * options.lineHeight
          )
        }
      }
    }
  }
})
