const { runAutomator, killWeChat } = require('./runAutomator')
const adb = require('./adb')
const runTimes = 60
let runCount = 0
let currentMode
const run = async mode => {
  if (currentMode === 'Photo' && mode === 'Live') return
  currentMode = mode
  const t0 = Date.now()
  const miniProgram = await launchMiniProgram()
  miniProgram.on('exception', error => {
    throw new Error(
      `memory test error, run ${runCount} times, execution time: ${parseInt(
        (Date.now() - t0) / 1000
      )}s, error: ${error.message}`
    )
  })
  await enterMakeup(miniProgram)
  while (runCount < runTimes) {
    console.log(`[${Date()}] memory test(${currentMode}) run ${runCount + 1} times`)
    await singleRun(miniProgram, mode)
    runCount++
  }
  await miniProgram.close()
  console.log(`memory test(${currentMode}) done, cost time: ${parseInt((Date.now() - t0) / 1000)}s`)
  runCount = 0
}
const launchMiniProgram = async () => {
  const miniProgram = await runAutomator()
  await new Promise((resolve, reject) => {
    adb.launchWeChat()
    const failLaunchTimeout = setTimeout(() => {
      reject(new Error(`launch miniProgram fail`))
    }, 120000)
    miniProgram
      .remote(true)
      .then(() => {
        clearTimeout(failLaunchTimeout)
        resolve()
      })
      .catch(e => {})
  })
  return miniProgram
}
const enterMakeup = async miniProgram => {
  // launcher page
  const page = await miniProgram.currentPage()
  const makeupBtn = await page.$('.action-btn')
  await makeupBtn.tap()
  await page.waitFor(500)
}
const singleRun = async (miniProgram, autoTestType = 'Live') => {
  // makeup - entry page
  let page = await miniProgram.currentPage()
  const mode = await page.$$('.action-btn')
  const modeIndex = autoTestType === 'Live' ? 0 : 1
  await mode[modeIndex].tap()
  await page.waitFor(500)
  // makeup - camera page
  // check init done
  let isCameraPage, isSDKReady
  // eslint-disable-next-line no-async-promise-executor
  await new Promise(async (resolve, reject) => {
    const initTimeout = setTimeout(() => {
      clearTimeout(initTimeout)
      reject(Error(`makeup sdk init timeout`))
    }, 60000)
    do {
      page = await miniProgram.currentPage().catch(e => ({}))
      isCameraPage = page.path === 'pages/makeup/camera/camera'
      if (isCameraPage) {
        isSDKReady = await page.data('isSDKReady').catch(e => false)
        await page.setData({ autoTestType })
      }
      await new Promise(resolve => setTimeout(resolve, 1000))
    } while (!isSDKReady)
    clearTimeout(initTimeout)
    resolve()
  })
  await page.waitFor(500)
  page = await miniProgram.currentPage()
  await page.$$('.try-button')
  await page.callMethod('applyMakeupByLook', 'BCC-29052_20200707_LK_01') // apply look

  // wait for apply done
  let isApplyFinish
  do {
    const tryOnList = await page.data('tryOnList')
    isApplyFinish = tryOnList.Look !== null
  } while (!isApplyFinish)
  await page.waitFor(500)
  await miniProgram.navigateBack()
}
try {
  adb.unlockDevice()
} catch (e) {
  if (e.message.includes('no devices/emulators found')) {
    console.log('no devices found, skip memory test. Please check device')
  }
}

const retryRun = async () => {
  let testDone = false
  let testSuccess = true
  do {
    await run('Live')
      .then(() => run('Photo'))
      .then(() => {
        testDone = true
      })
      .catch(e => {
        if (
          e.message.includes('launch miniProgram fail') ||
          e.message.includes('makeup sdk init timeout')
        ) {
          runCount = 0
          console.log(`[${Date()}] ${e.message}, retry.`)
        } else if (e.message.includes('Remote debug connection lost')) {
          const foregroundActivity = adb.getForegroundActivity()
          if (foregroundActivity === 'com.tencent.mm/.plugin.appbrand.ui.AppBrandUI00') {
            runCount = 0
            console.log(`[${Date()}] ${e.message} while miniProgram is not crashed, retry.`)
          } else {
            console.error(
              `memory test(${currentMode}) fail: ${e.message}(${foregroundActivity}), run ${runCount} times`
            )
            testDone = true
            testSuccess = false
          }
        } else {
          console.error(`memory test(${currentMode}) fail: ${e.message}, run ${runCount} times`)
          testDone = true
          testSuccess = false
        }
      })
  } while (!testDone)
  killWeChat()
  console.log(testSuccess ? '[Memory test] Success' : '[Memory test] Fail')
}
console.log(adb.getForegroundActivity())
retryRun()
