const logger = require('./logger')
const APP_PARAM = {
  domain: 'domain_config',
  dataType: 'dataType_config',
  skincareServer: 'skincareServer_config'
}

function setData(context, param) {
  return new Promise(resolve => {
    context.setData(param, () => {
      resolve()
    })
  })
}

function getImageInfo(param) {
  return new Promise((resolve, reject) => {
    wx.getImageInfo({
      ...param,
      success: res => {
        resolve(res)
      },
      fail: error => {
        reject(error)
      }
    })
  })
}

function putImageDataToCanvas(param, thisObj) {
  return new Promise((resolve, reject) => {
    wx.canvasPutImageData(
      {
        ...param,
        success: res => {
          resolve(res)
        },
        fail: e => {
          reject(e)
        }
      },
      thisObj
    )
  })
}

function toTempFilePathFromCanvas(param, thisObj) {
  return new Promise((resolve, reject) => {
    wx.canvasToTempFilePath(
      {
        ...param,
        success: res => {
          resolve(res.tempFilePath)
        },
        fail: e => {
          reject(e)
        }
      },
      thisObj
    )
  })
}

function saveImageToAlbum(param) {
  return new Promise((resolve, reject) => {
    wx.saveImageToPhotosAlbum({
      ...param,
      success: res => {
        resolve(res)
      },
      fail: error => {
        reject(error)
      }
    })
  })
}

function request(param) {
  return new Promise((resolve, reject) => {
    wx.request({
      ...param,
      success: res => {
        resolve(res)
      },
      fail: error => {
        reject(error)
      }
    })
  })
}
const requestWithRetry = async (params, logger = console, retryCount = 2) => {
  params.timeout = params.timeout || 10000
  logger.log(
    `starting request to ${params.url}` +
      (params.data ? `, query data: ${JSON.stringify(params.data)}` : '')
  )
  for (let i = 0; i <= retryCount; i++) {
    try {
      return await _request(params, logger)
    } catch (e) {
      logger.warn(`request fail in ${i}/${retryCount}th time, error: ${JSON.stringify(e)}`)
      if (i === retryCount) {
        throw new Error(JSON.stringify(e))
      }
    }
  }
}

const _request = (params, logger = console) => {
  return new Promise((resolve, reject) => {
    const start = Date.now()
    wx.request({
      ...params,
      success: res => {
        resolve(res)
      },
      fail: error => {
        reject(error)
      },
      complete: () => {
        logger.log(`request cost time: ${Date.now() - start}`)
      }
    })
  })
}

function setAppParam(options, isReset = false) {
  for (const [key, value] of Object.entries(APP_PARAM)) {
    if (options[key]) {
      setValue(value, options[key])
    } else if (isReset) {
      removeValue(value)
    }
  }

  logger.log(`setAppParam result: ${JSON.stringify(options)}`)
}

function setValue(key, value) {
  logger.log('setStorage:' + key + ':' + value)
  try {
    wx.setStorageSync(key, value)
  } catch (e) {
    logger.warn('setStorageSync' + key + ':' + value + ', Fail!', e)
  }
}

function getAppParam(options) {
  const configs = {}
  for (const [key, value] of Object.entries(APP_PARAM)) {
    const v = getValue(value)
    if (v) {
      configs[key] = v
    }
  }

  logger.log(`getAppParam result: ${JSON.stringify(configs)}`)
  return configs
}

function getValue(key, fallback) {
  try {
    const value = wx.getStorageSync(key)
    if (value) {
      // logger.log("getStorage:" + key + ":" + value);
      return value
    }
  } catch (e) {
    this.warn('getStorageSync, ' + key + ' Fail!', e)
  }
  // logger.log("getStorage:" + key + ":" + fallback + "(fallback)");
  return fallback
}

function removeValue(key) {
  logger.log('removeStorage:' + key)
  try {
    wx.removeStorageSync(key)
  } catch (e) {
    logger.warn('removeStorageSync' + key + ', Fail!', e)
  }
}

function removeHtmlTag(string = '') {
  return string.replace(/<[^>]*>?/gm, '')
}

const measureText = (text, fontSize = 10) => {
  text = String(text)
  const textSP = text.split('')
  let width = 0
  textSP.forEach(function(item) {
    if (/[a-zA-Z]/.test(item)) {
      width += 7
    } else if (/[0-9]/.test(item)) {
      width += 5.5
    } else if (/\./.test(item)) {
      width += 2.7
    } else if (/-/.test(item)) {
      width += 3.25
    } else if (/[\u4e00-\u9fa5]/.test(item)) {
      // Chinese word
      width += 10
    } else if (/\(|\)/.test(item)) {
      width += 3.73
    } else if (/\s/.test(item)) {
      width += 2.5
    } else if (/\\|\//.test(item)) {
      width += 2.78
    } else if (/%/.test(item)) {
      width += 8
    } else {
      width += 10
    }
  })
  return (width * fontSize) / 10
}

function showActionSheet(config) {
  const MAX_ITEM_LIST = 6
  if (config.itemList.length > MAX_ITEM_LIST) {
    const myConfig = {}
    for (const i in config) {
      myConfig[i] = config[i]
    }
    myConfig.page = 1
    myConfig.itemListBak = config.itemList
    myConfig.itemList = []
    const successFun = config.success
    myConfig.success = function(res) {
      if (res.tapIndex === 5) {
        myConfig.page++
        showActionSheet(myConfig)
      } else {
        res.tapIndex = res.tapIndex + 5 * (myConfig.page - 1)
        successFun(res)
      }
    }
    showActionSheet(myConfig)
  }
  if (!config.page) {
    wx.showActionSheet(config)
  } else {
    const page = config.page
    const itemListBak = config.itemListBak
    const itemList = []
    for (let i = 5 * (page - 1); i < 5 * page && i < itemListBak.length; i++) {
      itemList.push(itemListBak[i])
    }
    if (5 * page < itemListBak.length) {
      itemList.push('Next')
    }
    config.itemList = itemList
    wx.showActionSheet(config)
  }
}

const getImageDataFromUrl = (url, width, height) =>
  wait(1000).then(
    // wait 1s for preventing framework issue that osCanvas will be undefined.
    () =>
      new Promise(resolve => {
        const osCanvas = wx.createOffscreenCanvas({
          type: '2d',
          width,
          height
        })
        osCanvas.width = width
        osCanvas.height = height
        const ctx = osCanvas.getContext('2d')
        const image = osCanvas.createImage()
        image.onload = () => {
          ctx.drawImage(image, 0, 0, width, height)
          const imgData = ctx.getImageData(0, 0, width, height)
          resolve(imgData)
        }
        image.src = url
      })
  )

// can pass a custom ratio , but don't > systemInfo.pixelRatio
const getPixelRatio = ratio => (isNaN(parseInt(ratio)) ? wx.getWindowInfo().pixelRatio : ratio)

const wait = ms => new Promise(resolve => setTimeout(resolve, ms))

module.exports = {
  setData,
  getImageInfo,
  putImageDataToCanvas,
  toTempFilePathFromCanvas,
  saveImageToAlbum,
  request,
  requestWithRetry,
  setAppParam,
  getAppParam,
  getValue,
  setValue,
  removeHtmlTag,
  measureText,
  wait,
  showActionSheet,
  getImageDataFromUrl,
  getPixelRatio
}
